"""Miranda: an IRC bot"""

from __future__ import absolute_import, print_function, unicode_literals

import os
import argparse

from miranda.console import Console
from miranda.irc import IRC
from miranda.dispatch import listener, Listener, Namespace, Dispatch
from miranda.reactor import Reactor

__all__ = ['listener', 'Listener', 'Namespace', 'Miranda']
__author__ = "Sam Clements <sam@borntyping.co.uk>"
__version__ = "0.1.0"

class Miranda(object):
    """Ties the various objects together into a single interface"""
    
    def __init__(self, home, console=True):
        paths = [
            os.path.join(home, "modules"),
            os.path.join(os.path.dirname(__file__), "modules"),
        ]

        self.dispatch = Dispatch(paths)
        self.dispatch.load_module("example")
        self.dispatch.reload_module("example")
        
        connections = [
            IRC('irc.freenode.org', 6667)
        ]

        if console:
            connections.append(Console(miranda=self))
       
        self.reactor = Reactor(connections, self.dispatch)

    def go(self):
        self.reactor.go()
        return self

parser = argparse.ArgumentParser(prog="miranda")
parser.add_argument('home', nargs='?', default='~/.miranda',
    help="the directory containing configuration and plugins")
parser.add_argument('-c', '--console', action='store_true',
    help="start a Console connection")
parser.add_argument('-n', '--dry-run', action='store_true',
    help="halt before connecting")
parser.add_argument('-d', '--debug', action='store_true',
    help="print extra debug information")

def main():
    """Parses command line options to build and run a Miranda instance"""
    args = parser.parse_args()
    
    miranda = Miranda(os.path.expanduser(args.home), console=args.console)

    if args.debug:
        print(miranda.dispatch)
        print(miranda.reactor)

    if not args.dry_run:
        miranda.go() 
